// -----------------------------------------------------------------------------
//
// Copyright 2012(c) Analog Devices, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//  - Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  - Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  - Neither the name of Analog Devices, Inc. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//  - The use of this software may or may not infringe the patent rights
//    of one or more patent holders.  This license does not release you
//    from the requirement that you obtain separate licenses from these
//    patent holders to use this software.
//  - Use of the software either in source or binary form, must be run
//    on or directly connected to an Analog Devices Inc. component.
//
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
// FILE NAME : AD7327.v
// MODULE NAME : AD7327
// AUTHOR :acostina
// AUTHORS EMAIL : adrian.costina@analog.com
// -----------------------------------------------------------------------------
// SVN REVISION: 1529
// -----------------------------------------------------------------------------
// KEYWORDS : AD7327
// -----------------------------------------------------------------------------
// PURPOSE : Driver for the AD7327: 500 kSPS, 8-Channel, Software-Selectable, True
//           Bipolar Input, 12 Bit Plus Sign ADC
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Reset Strategy      : Active low reset signal
// Clock Domains       : fpga_clk_i 100 MHz
//                       adc_clk_i  10  MHz
// Critical Timing     : N/A
// Test Features       : N/A
// Asynchronous I/F    : N/A
// Instantiations      : N/A
// Synthesizable (y/n) : Y
// Target Device       : Cyclone IV E: EP4CE22F17C7 on BeMicroSDK kit
// Other               : The driver is intended to be used with AD7327
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

`timescale 1 ns / 100 ps //Use a timescale that is best for simulation.

//------------------------------------------------------------------------------
//----------- Module Declaration -----------------------------------------------
//------------------------------------------------------------------------------

module AD7327

//----------- Ports Declarations -----------------------------------------------
(
    //clock and reset signals
    input               fpga_clk_i,     // system clock
    input               adc_clk_i,      // clock to be applied to ADC to read the conversions results
    input               reset_i,        // active low reset signal

    //IP control and data interface
    input       [15:0]  data_i,         // data to be sent to the ADC
    input               wr_data_n_i,    // active low signal to initiate a data write to the ADC
    output reg  [15:0]  data_o,         // data read from the ADC
    output reg          data_rd_ready_o,// when set to high the data read from the ADC is available on the data_o bus
    output reg          data_wr_ready_o,// used to signal the status of an ADC write: 0 - write in progress, 1 - write complete

    //ADC control and data interface
    input               adc_sdo_i,      // ADC DOUT pin
    output reg          adc_sdi_o,      // ADC DIN pin
    output              adc_sclk_o,     // ADC SCLK pin
    output reg          adc_cs_n_o      // ADC \CS pin
);

//------------------------------------------------------------------------------
//----------- Registers Declarations -------------------------------------------
//------------------------------------------------------------------------------

reg [7:0]   adc_state;              // current state for the ADC control state machine
reg [7:0]   adc_next_state;         // next state for the ADC control state machine
reg [15:0]  data_i_reg ;            // register used on the fpga clock
reg [15:0]  data_adc;               // register used on the ADC clock
reg         adc_clk_en;             // clock enable signal
reg         sync_reg;               // register used to synchronize ADC clock with fpga clock
reg [31:0]  cycle_cnt;              // cycle time
reg [31:0]  sclk_cnt;               // count the sclk periods sent
reg [31:0]  delay_cs;               // registerused for increasing the time between CS Low and reading data
reg [31:0]  power_up;               // register used for the power_up sequence

//------------------------------------------------------------------------------
//----------- Local Parameters -------------------------------------------------
//------------------------------------------------------------------------------
// ADC states
localparam ADC_IDLE_STATE           = 6'b000001; // Default state
localparam ADC_START_CONV_STATE     = 6'b000010; // Start the conversion
localparam ADC_RW_DATA_STATE        = 6'b000100; // Generate clock signals for reading data from ADC
localparam ADC_DELAY_CS_STATE       = 6'b001000; // Adds the t11 delay
localparam ADC_TRANSFER_DATA_STATE  = 6'b010000; // Transfers data to the upper module
localparam ADC_WAIT_END_STATE       = 6'b100000; // Waits for the cycle time to end. Includes the tQuiet timing

// ADC timing
parameter real      FPGA_CLOCK_FREQ  = 100;    // FPGA clock frequency [MHz]
parameter real      ADC_CYCLE_TIME   = 2;  // minimum time between two ADC conversions  [us]

parameter [15:0]    ADC_CYCLE_CNT    = FPGA_CLOCK_FREQ * ADC_CYCLE_TIME - 2;

// ADC serial clock periods
parameter ADC_SCLK_PERIODS  = 5'd16; // number of clocks to be sent to the ADC to read the conversion result

//------------------------------------------------------------------------------
//------------------ Initial Block ---------------------------------------------
//------------------------------------------------------------------------------

initial
begin
    cycle_cnt       = ADC_CYCLE_CNT;
    sclk_cnt        = ADC_SCLK_PERIODS;
end

//------------------------------------------------------------------------------
//----------- Assign/Always Blocks ---------------------------------------------
//------------------------------------------------------------------------------

assign adc_sclk_o     =   sync_reg ? adc_clk_i: 1'b1 ;

// Clock synchronization process
// Makes sure that the clock enable signal is asserted during the high time of
// the ADC clock.
always @(posedge adc_clk_i)
begin
    if (reset_i == 1'b1)
    begin
        sync_reg    = 1'b0;
    end
    else
    begin
        sync_reg    = adc_clk_en;
    end
end

// Process for the timers used by the module
always @(posedge fpga_clk_i)
begin
    if(reset_i == 1'b1)
    begin
        cycle_cnt  <= ADC_CYCLE_CNT;
    end
    else
    begin
        if(adc_state == ADC_IDLE_STATE)
        begin
            cycle_cnt <= ADC_CYCLE_CNT ;
        end
        else if(cycle_cnt != 0)
        begin
            cycle_cnt <= cycle_cnt - 32'h1;
        end

        // timer used for respecting t2/t3 timings
        if (adc_state == ADC_IDLE_STATE)
        begin
            delay_cs    <= 32'h1;
        end
        else if (delay_cs != 32'h0)
        begin
            delay_cs    <= delay_cs - 32'h1;
        end
    end
end

// Read Data from the ADC
always @(negedge adc_clk_i)
begin
    if(sync_reg == 1'b1)
    begin
        data_o          <= { data_o[14:0], adc_sdo_i };
        sclk_cnt        <= sclk_cnt - 32'h1;
    end
    else
    begin
        sclk_cnt        <= ADC_SCLK_PERIODS;
    end
end

// Write Data to the ADC
always @(posedge adc_clk_i)
begin
    if(sync_reg == 1'b1)
    begin
        adc_sdi_o   <= data_adc [15];
        data_adc    <= {data_adc[14:0],1'b0};
    end
    else
    begin
        data_adc    <= {data_i_reg[14:0],1'b0 };
        if (adc_state == ADC_START_CONV_STATE || adc_state == ADC_RW_DATA_STATE)
        begin
            adc_sdi_o   <= data_i_reg [15];
        end
        else
        begin
            adc_sdi_o   <= 1'bz;
        end
    end
end

//update the ADC current state and the control signals
always @(posedge fpga_clk_i)
begin
    if(reset_i == 1'b1)
    begin
        adc_state   <= ADC_IDLE_STATE;
        data_i_reg  <= 16'h8000;    // Write control register, select channel 0 single ended for next conversion
                                    // power mode normal, use external reference, sequencer disabled
    end
    else
    begin
        adc_state <= adc_next_state;
        case (adc_state)
            ADC_IDLE_STATE:
            begin
                if( wr_data_n_i == 1'b0 )
                begin
                    data_i_reg      <= data_i;
                    data_wr_ready_o <= 1'b1;
                end
                else
                begin
                    data_wr_ready_o <= 1'b0;
                end
                adc_cs_n_o      <= 1'b1;
                data_rd_ready_o <= 1'b0;
                adc_clk_en      <= 1'b0;
            end
            ADC_START_CONV_STATE:
            begin
                adc_cs_n_o      <= 1'b0;
                data_wr_ready_o <= 1'b0;
                data_rd_ready_o <= 1'b0;
                adc_clk_en      <= 1'b0;
            end
            ADC_RW_DATA_STATE:
            begin
                adc_cs_n_o      <= 1'b0;
                data_wr_ready_o <= 1'b0;
                data_rd_ready_o <= 1'b0;
                adc_clk_en      <= 1'b1 ;
            end
            ADC_DELAY_CS_STATE:
            begin
                adc_cs_n_o      <= 1'b0;
                data_wr_ready_o <= 1'b0;
                data_rd_ready_o <= 1'b0;
                adc_clk_en      <= 1'b0;
            end
            ADC_TRANSFER_DATA_STATE:
            begin
                adc_cs_n_o      <= 1'b1;
                data_wr_ready_o <= 1'b0;
                data_rd_ready_o <= 1'b1;
                adc_clk_en      <= 1'b0;
            end
            ADC_WAIT_END_STATE:
            begin
                adc_cs_n_o      <= 1'b1;
                data_wr_ready_o <= 1'b0;
                data_rd_ready_o <= 1'b0;
                adc_clk_en      <= 1'b0;
            end
        endcase
    end
end

//update the ADC next state
always @(adc_state, delay_cs, sclk_cnt, cycle_cnt, sync_reg )
begin
    adc_next_state = adc_state;
    case (adc_state)
        ADC_IDLE_STATE:
        begin
            adc_next_state = ADC_START_CONV_STATE;
        end
        ADC_START_CONV_STATE:
        begin
            if (delay_cs == 32'h0 )
            begin
                adc_next_state = ADC_RW_DATA_STATE;
            end
        end
        ADC_RW_DATA_STATE:
        begin
             if(sclk_cnt == 32'h0)
            begin
                adc_next_state = ADC_DELAY_CS_STATE;
            end
        end
        ADC_DELAY_CS_STATE:
        begin
            if (sync_reg == 1'b0)
            begin
                adc_next_state = ADC_TRANSFER_DATA_STATE;
            end
        end
        ADC_TRANSFER_DATA_STATE:
        begin
            adc_next_state = ADC_WAIT_END_STATE;
        end
        ADC_WAIT_END_STATE:
        begin
            if ( cycle_cnt == 32'b0 )
            begin
                adc_next_state = ADC_IDLE_STATE;
            end
        end
        default:
        begin
            adc_next_state = ADC_IDLE_STATE;
        end
    endcase
end

endmodule
